package edu.umd.cs.voting.model;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import edu.umd.cs.voting.Controller;
import edu.umd.cs.voting.view.PartiesComponent;

public class Ballot {
	
	protected ArrayList pages;
	protected PartiesComponent partiesComponent;
	
	public Ballot(String file) {
		try {
			SAXBuilder builder = new SAXBuilder();
			Document document = builder.build(file);
			Element ballot = document.getRootElement();

			pages = new ArrayList();

			Controller.loadStyleInformation(ballot.getChild("style"));

			Element parties = ballot.getChild("parties");
			
			if (parties != null) {
				partiesComponent = new PartiesComponent(parties, Controller.CONTROLLER);
			}
			
			Iterator i = ballot.getChildren("page").iterator();
			while (i.hasNext()) {
				Element pageElement = (Element) i.next();
				Page page = new Page(this, pageElement);
				pages.add(page);
			}
						
		} catch (JDOMException e) {
			e.printStackTrace();
		}
	}
	
	public ArrayList getPages() {
		return pages;
	}
	
	public PartiesComponent getPartiesComponent() {
		return partiesComponent;
	}
	
	public void reset() {
		Iterator i = pages.iterator();
		while (i.hasNext()) {
			Item each = (Item) i.next();
			each.reset();
		}
	}
	
	public void submit() {		
		File voteResults = new File("vote_results.csv");
				
		StringBuffer results = new StringBuffer();

		if (!voteResults.exists()) {
			results.append("Date,");
			
			Iterator i = pages.iterator();
			while (i.hasNext()) {
				Page eachPage = (Page) i.next();
	
				Iterator j = eachPage.items.iterator();
				while (j.hasNext()) {
					Item each = (Item) j.next();
					for (int k = 0; k < each.getMaxSelectedCandidates(); k++) {
						results.append(each.getTitle().replaceAll("\n", " ").replaceAll(",", " "));
						results.append(",");
					}
				}
			}
		}

		results.append("\r\n");
		results.append(new Date().toString());
		results.append(",");
		
		Iterator i = pages.iterator();
		while (i.hasNext()) {
			Page eachPage = (Page) i.next();
			
			Iterator j = eachPage.items.iterator();
			while (j.hasNext()) {
				int selectedCandidates = 0;
				Item each = (Item) j.next();
				//results.append(each.getTitle());
					
				ArrayList candidates = each.getCandidates();
				for (int k = 0; k < candidates.size(); k++) {
					Candidate candidate = (Candidate) candidates.get(k);
					if (candidate.isSelected()) {
						selectedCandidates++;
						results.append(candidate.getName().replaceAll("\n", " ").replaceAll(",", " "));
						results.append(",");
					}
				}
				
				while (selectedCandidates < each.getMaxSelectedCandidates()) {
					results.append(",");
					selectedCandidates++;
				}	
			}
		}

		try {
			BufferedWriter out = new BufferedWriter(new FileWriter(voteResults, true));
			out.write(results.toString());
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		System.out.println(results.toString());
	}	
}
