package edu.umd.cs.voting.view;

import java.awt.Color;
import java.awt.Font;
import java.util.Iterator;

import org.jdom.DataConversionException;
import org.jdom.Element;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.activities.PActivity;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PImage;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolox.event.PNotification;
import edu.umd.cs.piccolox.event.PNotificationCenter;
import edu.umd.cs.voting.Controller;
import edu.umd.cs.voting.model.Candidate;

public class CandidateNode extends PNode {

	protected static Font CANDIDATE_NAME_FONT = new Font("Helvetica", Font.PLAIN, 12);
	protected static Font PARTY_NAME_FONT = new Font("Helvetica", Font.ITALIC, 10);
	protected static Color BACKGROUND = Color.white;
	protected static Color FOREGROUND = Color.black;
	protected static Color BACKGROUND_SELECTED = Color.black;
	protected static Color BACKGROUND_REVIEW_MODE_SELECTED = Color.black;
	protected static Color FOREGROUND_SELECTED = Color.white;
	protected static Color BACKGROUND_UNDERVOTE = null;//new Color(0.8f, 0.3f, 0.3f);
	protected static Color BACKGROUND_REVIEW_MODE_UNDERVOTE = null;//new Color(0.8f, 0.3f, 0.3f);
	protected static double PADDING = 5;
	protected static double MARGIN = 1;
	protected static double SPACING = 1;
	protected static double CHECK_BOX_SIZE = 20;
	protected static boolean IS_ANIMATING = false;

	protected Candidate candidate;
	
	protected PText name;
	protected PText otherName;
	protected PText party;
	protected Checkbox checkbox;

	public CandidateNode(Candidate candidate) {

		this.candidate = candidate;
		
		if (candidate.getName() != null) {
			name = new PText(candidate.getName());
			name.setFont(CANDIDATE_NAME_FONT);
			name.setTextPaint(FOREGROUND);
			addChild(name);
		}
		
		if (candidate.getOtherName() != null) {
			otherName = new PText(candidate.getOtherName());
			otherName.setFont(CANDIDATE_NAME_FONT);
			otherName.setTextPaint(FOREGROUND);
			addChild(otherName);
		}

		if (candidate.getParty() != null) {
			party = new PText(candidate.getParty());
			party.setFont(PARTY_NAME_FONT);
			party.setTextPaint(FOREGROUND);
			addChild(party);
		}
		
		double xOffset = PADDING;
		double yOffset = PADDING;
		
		if (name != null) {
			name.offset(xOffset, yOffset);
			yOffset += name.getHeight() + SPACING;
		}
		
		if (otherName != null) {
			otherName.offset(xOffset, yOffset);
			yOffset += otherName.getHeight() + SPACING;
		}
		
		if (party != null) {
			party.offset(xOffset, yOffset);
			yOffset += party.getHeight() + SPACING;
		}

		checkbox = new Checkbox();
		addChild(checkbox);
		checkbox.setBounds(PADDING, PADDING, CHECK_BOX_SIZE + PADDING, CHECK_BOX_SIZE + PADDING);
			
		getUnionOfChildrenBounds(getBoundsReference());
		getBoundsReference().inset(-PADDING, -PADDING);
		
		setPaint(BACKGROUND);
				
		addInputEventListener(new PBasicInputEventHandler() {
			public void mousePressed(PInputEvent event) {
				event.setHandled(true);
			}
			public void mouseReleased(PInputEvent event) {
				if (IS_ANIMATING) return;
				
				if (CandidateNode.this.candidate.isSelected()) {
					CandidateNode.this.candidate.getItem().unselect(CandidateNode.this.candidate);
				} else {
					if (CandidateNode.this.candidate.getItem().canSelectAnother()) {
						CandidateNode.this.candidate.getItem().select(CandidateNode.this.candidate);
					} else {
						animateSelectionFrom(CandidateNode.this.candidate.getItem().getFirstSelected());
					}
				}
				event.setHandled(true);
			}
		});
		
		PNotificationCenter.defaultCenter().addListener(this, "candidateSelectedStateChanged", Candidate.SELECTED_STATE_CHANGED, candidate);
	}
	
	public void setName(String name) {
		this.name.setText(name);
		candidate.setName(name);
	}

	public boolean setBounds(double x, double y, double width, double height) {
		if (super.setBounds(x, y, width, height)) {
			updateCheckboxBounds();
			return true;
		}
		return false;
	}
	
	public void updateCheckboxBounds() {
		PBounds b = getBoundsReference();
		double x = b.getX() + b.getWidth() - CHECK_BOX_SIZE - CHECK_BOX_SIZE;
		double y = b.getY() + ((b.getHeight() - CHECK_BOX_SIZE) / 2);	
		checkbox.setBounds(x, y, CHECK_BOX_SIZE, CHECK_BOX_SIZE);
	}
	
	public Checkbox getCheckbox() {
		return checkbox;
	}
	
	public void candidateSelectedStateChanged(PNotification notification) {
		if (candidate.isSelected()) {
			if (name != null) name.setTextPaint(FOREGROUND_SELECTED);
			if (otherName != null) otherName.setTextPaint(FOREGROUND_SELECTED);
			if (party != null) {
				party.setTextPaint(FOREGROUND_SELECTED);
				if (party.getText().equalsIgnoreCase("Write-In")) {
					new ScreenKeyboard(Controller.CONTROLLER, this);
				}
				
				if (candidate.getName().equalsIgnoreCase("Write-In")) {
					candidate.getItem().unselect(candidate);
					return;
				}
			}
			setPaint(BACKGROUND_SELECTED);
			checkbox.setSelected(true);
		} else {
			if (name != null) name.setTextPaint(FOREGROUND);
			if (otherName != null) otherName.setTextPaint(FOREGROUND);
			if (party != null) party.setTextPaint(FOREGROUND);
			setPaint(BACKGROUND);
			checkbox.setSelected(false);
		}
		
		ItemNode item = (ItemNode) getParent();

		Iterator i = item.candidates.iterator();
		while (i.hasNext()) {
			CandidateNode each = (CandidateNode) i.next();
			if (each.candidate.getParty() != null && 
				each.candidate.getParty().equalsIgnoreCase("Write-In")) {
				
				if (!each.candidate.isSelected()) {
					each.setName("Write-In");
				}
			}
		}
		
		item.setHasFocus(!item.getHasFocus());
		item.setHasFocus(!item.getHasFocus());
	}	

	public void animateSelectionFrom(final Candidate source) {
		IS_ANIMATING = true;

		CandidateNode selected = ((ItemNode)getParent()).getCandidateNodeFromCandidate(source);
		Checkbox selectedCheckBox = selected.getCheckbox();
		final PImage image = new PImage(selectedCheckBox.toImage(), false);
		getParent().addChild(image);
		
		PBounds b1 = selectedCheckBox.getGlobalFullBounds();
		PBounds b2 = checkbox.getGlobalFullBounds();
		double distance = b1.getCenter2D().distance(b2.getCenter2D());
		image.globalToLocal(b1);
		image.globalToLocal(b2);

 		image.setOffset(b1.x, b1.y);
		
		PActivity a = image.animateToPositionScaleRotation(b2.x, b2.y, 1, 0, (long) distance * 4 );
		a.setDelegate(new PActivity.PActivityDelegate() {
			public void activityStarted(PActivity activity) {
				source.getItem().unselect(source);
			}
			public void activityStepped(PActivity activity) {
			}
			public void activityFinished(PActivity activity) {
				candidate.getItem().select(candidate);
				image.removeFromParent();
				IS_ANIMATING = false;
			}
		});
	}

	public Candidate getCandidate() {
		return candidate;
	}

	public static void loadStyleInformation(Element style) {
		try {
			CandidateNode.BACKGROUND = Controller.readColorStyle(style.getAttributeValue("background"));
			CandidateNode.BACKGROUND_SELECTED = Controller.readColorStyle(style.getAttributeValue("selectedbackground"));
			CandidateNode.BACKGROUND_REVIEW_MODE_SELECTED = Controller.readColorStyle(style.getAttributeValue("reviewModeSelectedbackground"));
			CandidateNode.BACKGROUND_UNDERVOTE = Controller.readColorStyle(style.getAttributeValue("undervoteBackground"));
			CandidateNode.BACKGROUND_REVIEW_MODE_UNDERVOTE = Controller.readColorStyle(style.getAttributeValue("reviewModeUndervoteBackground"));
			CandidateNode.FOREGROUND = Controller.readColorStyle(style.getAttributeValue("foreground"));
			CandidateNode.FOREGROUND_SELECTED = Controller.readColorStyle(style.getAttributeValue("selectedforeground"));
			CandidateNode.CANDIDATE_NAME_FONT = Controller.readFontStyle(style.getChild("name"));
			CandidateNode.PARTY_NAME_FONT = Controller.readFontStyle(style.getChild("party"));

			Element candidateLayout = style.getChild("layout");
			CandidateNode.MARGIN = candidateLayout.getAttribute("margin").getFloatValue();
			CandidateNode.PADDING = candidateLayout.getAttribute("padding").getFloatValue();
		} catch (DataConversionException e) {
			e.printStackTrace();
		}
	}	
}
