package edu.umd.cs.voting.view;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Iterator;

import org.jdom.DataConversionException;
import org.jdom.Element;

import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PNodeFilter;
import edu.umd.cs.voting.Controller;
import edu.umd.cs.voting.model.Candidate;
import edu.umd.cs.voting.model.Item;

public class ItemNode extends PNode {

	protected static Font TITLE_FONT = new Font("Helvetica", Font.BOLD, 16);
	protected static Font OVERVIEW_TITLE_FONT = new Font("Helvetica", Font.BOLD, 16);
	protected static Font OVERVIEW_CANDIDATE_FONT = new Font("Helvetica", Font.PLAIN, 16);
	protected static Font SUBTITLE_FONT = new Font("Helvetica", Font.ITALIC, 12);
	protected static Font QUESTION_FONT = new Font("Helvetica", Font.PLAIN, 12);
	protected static Color FADED_TEXT_COLOR = Color.GRAY;
	protected static Color FADED_BACKGROUND_COLOR = Color.LIGHT_GRAY.brighter();
	protected static double PADDING = 5;
	protected static double MARGIN = 1;

	protected Item item;

	protected PText title;
	protected PText subtitle;
	protected PText question;
	protected ArrayList candidates;
	protected PNode overviewBackground;
	protected boolean hasFocus = true;
	
	public ItemNode(Item item) {
		this.item = item;
		
		title = new PText(item.getTitle());
		title.setPaint(Color.WHITE);
		title.setFont(TITLE_FONT);
		title.setCenteredText(true);
		title.setPickable(false);
		addChild(title);
		
		subtitle = new PText(item.getSubtitle());
		subtitle.setPaint(Color.WHITE);
		subtitle.setFont(SUBTITLE_FONT);
		subtitle.setCenteredText(true);
		subtitle.setPickable(false);
		addChild(subtitle);
		
		if (item.getQuestion() != null) {
			question = new PText(item.getQuestion());
			question.setPaint(Color.WHITE);
			question.setFont(QUESTION_FONT);		
			question.setPickable(false);
			addChild(question);
		}
		
		Iterator i = item.getCandidates().iterator();
		while (i.hasNext()) {
			if (candidates == null) candidates = new ArrayList();
			Candidate each = (Candidate) i.next();
			CandidateNode node = new CandidateNode(each);
			candidates.add(node);
			addChild(node);
		}
		
		setPaint(Color.BLACK);
		
		addInputEventListener(new PBasicInputEventHandler() {
			public void mouseReleased(PInputEvent event) {
				event.setHandled(true);
			}
			public void mousePressed(PInputEvent event) {
				Controller.CONTROLLER.setFocusNode(ItemNode.this);
				event.setHandled(true);
//				ItemNode.this.setHasFocus(!hasFocus);
			}
		});		
	}
	
	public void performOfficeBlockLayout(double width) {
		width -= ((PADDING + CandidateNode.MARGIN) * 2);
		
		double xOffset = PADDING + CandidateNode.MARGIN;
		double yOffset = PADDING + CandidateNode.MARGIN;

		title.setConstrainWidthToTextWidth(false);
		subtitle.setConstrainWidthToTextWidth(false);
		if (question != null) question.setConstrainWidthToTextWidth(false);

		title.setWidth(width);
		subtitle.setWidth(width);
		if (question != null) question.setWidth(width);
		
		title.offset(xOffset, yOffset);
		yOffset += title.getHeight() + CandidateNode.MARGIN;

		subtitle.offset(xOffset, yOffset);
		yOffset += subtitle.getHeight() + CandidateNode.MARGIN;

		if (question != null) {
			question.offset(xOffset, yOffset);
			yOffset += question.getHeight() + CandidateNode.MARGIN;
		}

		Iterator i = candidates.iterator();
		while (i.hasNext()) {
			CandidateNode each = (CandidateNode) i.next();
			each.offset(xOffset, yOffset);
			each.setWidth(width);
			yOffset += each.getHeight() + CandidateNode.MARGIN;
		}

		getUnionOfChildrenBounds(getBoundsReference());
				
		getBoundsReference().inset(-(PADDING + CandidateNode.MARGIN), -(PADDING + CandidateNode.MARGIN));
		
		if (item.height != 0){
			PNode n = new PNode();
			
			n.setPaint(Color.WHITE);
			n.setBounds(xOffset, yOffset, width, item.height - getBoundsReference().height);
//			n.getBoundsReference().inset(PADDING, PADDING);
	//		n.getBoundsReference().y -= PADDING;
			getBoundsReference().height = item.height;
			addChild(n);
		}
		
		setHasFocus(false);
	}
	
	public void performPartyColumnLayout(double width) {
		if (question != null) {
			performPartyColumnReferendumLayout(width);
		} else {
			width -= ((PADDING + CandidateNode.MARGIN) * 2);
	
			double xOffset = PADDING + CandidateNode.MARGIN;
			double yOffset = PADDING + CandidateNode.MARGIN;
	
			double titleSubtitleWidth = title.getWidth() + CandidateNode.MARGIN + subtitle.getWidth();
			double tempXOffset = xOffset + ((width - titleSubtitleWidth) / 2);
			
			title.offset(tempXOffset, yOffset);
			subtitle.offset(tempXOffset + CandidateNode.MARGIN + title.getWidth(), yOffset);
	
			yOffset += Math.max(title.getHeight(), subtitle.getHeight()) + CandidateNode.MARGIN;
	
			if (question != null) {
				question.offset(xOffset, yOffset);
				yOffset += question.getHeight() + CandidateNode.MARGIN;
			}
	
			Iterator i = candidates.iterator();
			while (i.hasNext()) {
				CandidateNode each = (CandidateNode) i.next();
				each.offset(xOffset, yOffset);
				each.setWidth(width);
				yOffset += each.getHeight() + CandidateNode.MARGIN;
			}	
		}
		
		getUnionOfChildrenBounds(getBoundsReference());
		getBoundsReference().inset(-(PADDING + CandidateNode.MARGIN), -(PADDING + CandidateNode.MARGIN));		
	}

	public void performPartyColumnReferendumLayout(double width) {
		width -= ((PADDING + CandidateNode.MARGIN) * 2);

		double xOffset = PADDING + CandidateNode.MARGIN;
		double yOffset = PADDING + CandidateNode.MARGIN;

		double titleSubtitleWidth = title.getWidth() + CandidateNode.MARGIN + subtitle.getWidth();
		double tempXOffset = xOffset + ((width - titleSubtitleWidth) / 2);

		title.offset(tempXOffset, yOffset);
		subtitle.offset(tempXOffset + CandidateNode.MARGIN + title.getWidth(), yOffset);

		yOffset += Math.max(title.getHeight(), subtitle.getHeight()) + CandidateNode.MARGIN;

		question.setConstrainWidthToTextWidth(false);
		question.offset(xOffset, yOffset);
		question.setWidth(width);
		
		yOffset += question.getHeight() + CandidateNode.MARGIN;
		
		PNode yes =  (PNode) candidates.get(0);
		PNode no =  (PNode) candidates.get(1);
		
		yes.offset(xOffset, yOffset);
		yes.setWidth(width / 2);
		no.offset(xOffset + width / 2, yOffset);
	}
	
	public CandidateNode getCandidateNodeFromCandidate(Candidate candidate) {
		Iterator i = candidates.iterator();
		while (i.hasNext()) {
			CandidateNode each = (CandidateNode) i.next();
			if (each.getCandidate() == candidate) {
				return each;
			}
		}		
		return null;
	}

	public boolean getPageBreak() {
		return item.getPageBreak();
	}
	
	public boolean getHasFocus() {
		return hasFocus;
	}
	public void setHasFocus(boolean hasFocus) {
		if (this.hasFocus != hasFocus) {
			this.hasFocus = hasFocus;
			
			if (hasFocus || getParent() instanceof PCamera) {
				if (overviewBackground != null) {
					removeChild(overviewBackground);
					overviewBackground = null;
				}
			} else {
				if (overviewBackground == null) {
					double yOffset = 0;
					PPath border;
					
					overviewBackground = new PNode();
					overviewBackground.setPaint(CandidateNode.BACKGROUND);
					overviewBackground.setBounds(getBoundsReference());
					overviewBackground.setBounds(overviewBackground.getBoundsReference().inset(1, 1));
					overviewBackground.setChildrenPickable(false);

					PNode overview = new PNode();

					PText titleCopy = new PText(title.getText());
					titleCopy.setFont(OVERVIEW_TITLE_FONT);
					titleCopy.setConstrainWidthToTextWidth(false);
					titleCopy.setConstrainHeightToTextHeight(true);
					titleCopy.setCenteredText(true);
					titleCopy.setWidth(getWidth());
					overview.addChild(titleCopy);
					yOffset += titleCopy.getHeight() + 5;
					
					ArrayList selectedCandidates = new ArrayList();
					Iterator i = candidates.iterator();
					while (i.hasNext()) {
						CandidateNode each = (CandidateNode) i.next();
						if (each.candidate.isSelected()) {
							selectedCandidates.add(each);
						}
					}					

					if (true) {
//					if (selectedCandidates.size() > 0) {
						int underVoted = item.getMaxSelectedCandidates() - selectedCandidates.size();
						
						while (underVoted > 0) {
							selectedCandidates.add("Not Voted");
							underVoted--;
						}
						
						i = selectedCandidates.iterator();						
						while (i.hasNext()) {
							Object each = i.next();
							String nameString = null;
							String otherNameString = null;
							boolean isUndervote = false;
							
							if (each instanceof CandidateNode) {								
								nameString = ((CandidateNode)each).candidate.getName();
								otherNameString = ((CandidateNode)each).candidate.getOtherName();
							} else {
								nameString = (String) each;
								isUndervote = true;
							}
							
							PText name = new PText(nameString);
							name.setFont(OVERVIEW_CANDIDATE_FONT);
							name.setCenteredText(true);
							name.setConstrainWidthToTextWidth(false);
							name.setConstrainHeightToTextHeight(true);
							name.offset(0, yOffset);

							if (isUndervote) {
								if (Controller.IS_REVIEW_MODE) {
									name.setPaint(CandidateNode.BACKGROUND_REVIEW_MODE_UNDERVOTE);
								} else {
									name.setPaint(CandidateNode.BACKGROUND_UNDERVOTE);
								}
							} else {
								if (Controller.IS_REVIEW_MODE) {
									name.setPaint(CandidateNode.BACKGROUND_REVIEW_MODE_SELECTED);
								} else {								
									name.setPaint(CandidateNode.BACKGROUND_SELECTED);
								}
							}

							name.setTextPaint(CandidateNode.FOREGROUND_SELECTED);
							name.setWidth(getWidth());
							name.setBounds(name.getBounds().inset(20, 0));

							border = PPath.createRectangle(0, 0, 1, 1);
							border.setPaint(null);
							border.setStrokePaint(Color.BLACK);
							border.setBounds(name.getFullBounds().inset(-2, -2));

							overview.addChild(border);							
							yOffset += name.getHeight() + 5;
							overview.addChild(name);

							if (otherNameString != null) {
								name = new PText(otherNameString);
								name.setFont(OVERVIEW_CANDIDATE_FONT);
								name.setCenteredText(true);
								name.setConstrainWidthToTextWidth(false);
								name.setConstrainHeightToTextHeight(true);
								name.setPaint(CandidateNode.BACKGROUND_SELECTED);
								name.setTextPaint(CandidateNode.FOREGROUND_SELECTED);
								name.offset(0, yOffset);
								name.setWidth(getWidth());
								name.setBounds(name.getBounds().inset(20, 0));

								border = PPath.createRectangle(0, 0, 1, 1);
								border.setPaint(null);
								border.setStrokePaint(Color.BLACK);
								border.setBounds(name.getFullBounds().inset(-2, -2));
								
								overview.addChild(border);
								yOffset += name.getHeight() + 5;
								overview.addChild(name);	
							}
						}
					}
					
					overview.setBounds(overview.getUnionOfChildrenBounds(overview.getBoundsReference()));
					overviewBackground.addChild(overview);
					overview.centerFullBoundsOnPoint(overviewBackground.getBoundsReference().getCenterX(), overviewBackground.getBoundsReference().getCenterY());
					addChild(overviewBackground);
				}
			}
		}
				
		if (title != null) title.setVisible(hasFocus);
		if (subtitle != null) subtitle.setVisible(hasFocus);
		if (question != null) question.setVisible(hasFocus);
		
		Iterator i = candidates.iterator();
		while (i.hasNext()) {
			CandidateNode each = (CandidateNode) i.next();
			each.setVisible(hasFocus);
		}		
	}

	PNodeFilter filter = new PNodeFilter() {
		public boolean accept(PNode aNode) { return aNode instanceof PText; }
		public boolean acceptChildrenOf(PNode aNode) { return true; }
	};
		
	public void setFaded(boolean faded) {
		if (faded) {
			if (overviewBackground != null) {

				Iterator i = overviewBackground.getAllNodes(filter, null).iterator();
				while (i.hasNext()) {
					PText each = (PText) i.next();
					each.setTextPaint(FADED_TEXT_COLOR);
				}
			
				Controller.schedualNewActivityFor(overviewBackground, overviewBackground.animateToColor(FADED_BACKGROUND_COLOR, Controller.ANIMATION_TIME));
//				overviewBackground.setPaint(FADED_BACKGROUND_COLOR);
			}
		} else {
			Iterator i = overviewBackground.getAllNodes(filter, null).iterator();
			while (i.hasNext()) {
				PText each = (PText) i.next();
				each.setTextPaint(Color.BLACK);
			}
			
			if (overviewBackground != null) {
				Controller.schedualNewActivityFor(overviewBackground, overviewBackground.animateToColor(Color.WHITE, Controller.ANIMATION_TIME));
//				overviewBackground.setPaint(Color.WHITE);
			}
		}
	}
	
	public void setFullFaded(boolean faded) {
		setFaded(faded);
	}
	
	public void clearBallot() {
		Iterator i = candidates.iterator();
		while (i.hasNext()) {
			CandidateNode each = (CandidateNode) i.next();
			item.unselect(each.candidate);
			if ((each.party != null) && (each.party.getText() != null)) {
				if (each.party.getText().equalsIgnoreCase("Write-In")) {
					each.setName("Write-In");
				}
			}
		}
		setHasFocus(true);
		setHasFocus(false);
	}

	public static void loadStyleInformation(Element style) {
		try {
			Element itemLayout = style.getChild("layout");
			ItemNode.FADED_BACKGROUND_COLOR = Controller.readColorStyle(style.getAttributeValue("fadedbackground"));
			ItemNode.FADED_TEXT_COLOR = Controller.readColorStyle(style.getAttributeValue("fadedtext"));
			ItemNode.MARGIN = itemLayout.getAttribute("margin").getFloatValue();
			ItemNode.PADDING = itemLayout.getAttribute("padding").getFloatValue();			
			ItemNode.TITLE_FONT = Controller.readFontStyle(style.getChild("title"));
			ItemNode.OVERVIEW_TITLE_FONT = Controller.readFontStyle(style.getChild("overviewtitle"));
			ItemNode.OVERVIEW_CANDIDATE_FONT = Controller.readFontStyle(style.getChild("overviewcandidate"));
			ItemNode.SUBTITLE_FONT = Controller.readFontStyle(style.getChild("subtitle"));
			ItemNode.QUESTION_FONT = Controller.readFontStyle(style.getChild("question"));
		} catch (DataConversionException e) {
			e.printStackTrace();
		}
	}	
}
