package edu.umd.cs.voting.view;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Iterator;

import org.jdom.DataConversionException;
import org.jdom.Element;

import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.voting.Controller;
import edu.umd.cs.voting.model.Ballot;
import edu.umd.cs.voting.model.Page;

public class BallotNode extends PNode {

	protected static Color BACKGROUND_COLOR = Color.LIGHT_GRAY;
	protected static Font TITLE_FONT = new Font("Helvetica", Font.BOLD, 24);
	protected static Font INSTRUCTIONS_FONT = new Font("Helvetica", Font.PLAIN, 12);
	public static double WIDTH = 768;
	public static double HEIGHT = 1024;
	public static boolean FULL_SCREEN_MODE = false;
	public static boolean HIDE_CURSOR = false;
	protected static double PADDING = 5;
	protected static double MARGIN = 1;
	protected static boolean OFFICE_BLOCK = true;
	protected Controller controller;
	protected Ballot ballot;
	protected ArrayList pages;
	protected NavigationPanelNode navigationPanel;

	public BallotNode(Ballot ballot, Controller controller) {
		this.ballot = ballot;
		this.controller = controller;

		ArrayList pageThumbnails = new ArrayList();
				
		pages = new ArrayList();
		Iterator i = ballot.getPages().iterator();
		while (i.hasNext()) {
			PageNode each = new PageNode((Page)i.next());
			addChild(each);
			pages.add(each);
			pageThumbnails.add(new ButtonPageNode(each));
		}

		navigationPanel = new NavigationPanelNode(controller.getNavigationCanvas(), pageThumbnails);
		
		layoutVertically(PageNode.PAGE_WIDTH);

		final PCamera camera = controller.getCanvas().getCamera();
		camera.setPaint(BACKGROUND_COLOR);
		
		controller.getCanvas().addInputEventListener(new PBasicInputEventHandler() {
			public void mousePressed(PInputEvent event) {
				camera.setPaint(((Color)camera.getPaint()).darker());
				event.setHandled(true);
			}
			public void mouseReleased(PInputEvent event) {
				camera.setPaint(((Color)camera.getPaint()).brighter());
				Controller.CONTROLLER.setFocusNode(BallotNode.this);
				event.setHandled(true);
			}
		});		
	}

	public void layoutVertically(double width) {
		width -= ((PADDING + PageNode.MARGIN) * 2);

		double xOffset = PADDING + PageNode.MARGIN;
		double yOffset = PADDING + PageNode.MARGIN;
		
		Iterator i = pages.iterator();
		while (i.hasNext()) {
			PageNode each = (PageNode) i.next();
			if (OFFICE_BLOCK) {
				each.performOfficeBlockLayout(width);
			} else {
				each.performPartyColumnLayout(width);
			}
			each.offset(xOffset, yOffset);
			xOffset += each.getWidth() + PageNode.MARGIN;
		}
		
		getUnionOfChildrenBounds(getBoundsReference());
		getBoundsReference().inset(-(PADDING + PageNode.MARGIN), -(PADDING + PageNode.MARGIN));
	}

	public void layoutHorizontally(double width) {
	}	
		
	public PNode nextFocus() {
		return nextFocus(Controller.CONTROLLER.getFocusNode());
	}
	
	public PNode nextFocus(PNode focus) {		
		if (focus == this) {
			return (PNode) pages.get(0);
		}
		
		if (focus instanceof PageNode) {
			PageNode pageNode = (PageNode) focus;
			if (!pageNode.items.isEmpty()) {
				return (PNode) pageNode.items.get(0);
			} else {
				if (nextPage(pageNode) != pageNode) {
					return nextFocus(nextPage(pageNode));
				}				
				return nextPage(pageNode);
			}
		}
		
		if (focus instanceof ItemNode) {
			ItemNode itemNode = (ItemNode) focus;
			PageNode pageNode = (PageNode) itemNode.getParent();

			int index = pageNode.items.indexOf(itemNode);
			if (index < pageNode.items.size() - 1) {
				return (ItemNode) pageNode.items.get(index + 1);
			}

			if (nextPage(pageNode) != pageNode) {
				return nextFocus(nextPage(pageNode));
			}

			return itemNode;
		}
		
		return null;
	}
	
	public PageNode nextPage(PageNode pageNode) {
		int index = pages.indexOf(pageNode);
		if (index < pages.size() - 1) {
			return (PageNode) pages.get(index + 1);
		}
		return pageNode;
	}	

	public PNode previousFocus() {
		return previousFocus(Controller.CONTROLLER.getFocusNode());
	}
			
	public PNode previousFocus(PNode focus) {
		if (focus == this) {
			return this;
		}

		if (focus instanceof PageNode) {
			PageNode pageNode = (PageNode) focus;
			if (pageNode == previousPage(pageNode)) {
				return this;
			} else {
				PageNode previousPage = previousPage(pageNode);
				if (!previousPage.items.isEmpty()) {
					return (PNode) previousPage.items.get(previousPage.items.size() - 1);
				}
				return previousPage;
			}			
		}

		if (focus instanceof ItemNode) {
			ItemNode itemNode = (ItemNode) focus;
			PageNode pageNode = (PageNode) itemNode.getParent();

			int index = pageNode.items.indexOf(itemNode);
			if (index > 0) {
				return (ItemNode) pageNode.items.get(index - 1);
			}

			if (previousPage(pageNode) != pageNode) {
				return previousFocus(pageNode);
			}

			return itemNode;
		}

		return focus;		
	}

	public PageNode previousPage(PageNode pageNode) {
		int index = pages.indexOf(pageNode);
		if (index > 0) {
			return (PageNode) pages.get(index - 1);
		}
		return pageNode;
	}

	public void setFaded(boolean faded) {
		if (faded) {
			// use faded colors
		} else {
			// use normal colors
		}
	}
	
	public void setFullFaded(boolean faded) {
		setFaded(faded);
		
		Iterator i = pages.iterator();
		while (i.hasNext()) {
			PageNode each = (PageNode) i.next();
			each.setFullFaded(faded);
		}
	}
	
	public void clearBallot() {
		Iterator i = pages.iterator();
		while (i.hasNext()) {
			PageNode each = (PageNode) i.next();
			each.clearBallot();
		}
	}
	
	public static void loadStyleInformation(Element style) {
		try {			
			BallotNode.BACKGROUND_COLOR = Controller.readColorStyle(style.getAttributeValue("background"));
			Element ballotLayout = style.getChild("layout");
			BallotNode.MARGIN = ballotLayout.getAttribute("margin").getFloatValue();
			BallotNode.PADDING = ballotLayout.getAttribute("padding").getFloatValue();
			BallotNode.WIDTH = ballotLayout.getAttribute("width").getFloatValue();
			BallotNode.HEIGHT = ballotLayout.getAttribute("height").getFloatValue();
			BallotNode.FULL_SCREEN_MODE = ballotLayout.getAttribute("fullscreenmode").getBooleanValue();
			BallotNode.HIDE_CURSOR = ballotLayout.getAttribute("hideCursor").getBooleanValue();
			BallotNode.OFFICE_BLOCK = ballotLayout.getAttributeValue("type").equals("officeblock");
			BallotNode.TITLE_FONT = Controller.readFontStyle(style.getChild("title"));
			BallotNode.INSTRUCTIONS_FONT = Controller.readFontStyle(style.getChild("instructions"));
		} catch (DataConversionException e) {
			e.printStackTrace();
		}
	}	
}
