package edu.umd.cs.voting.view;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Iterator;

import org.jdom.DataConversionException;
import org.jdom.Element;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.util.PNodeFilter;
import edu.umd.cs.voting.Controller;
import edu.umd.cs.voting.model.Item;
import edu.umd.cs.voting.model.Page;

public class PageNode extends PNode {

	protected static Font TITLE_FONT = new Font("Helvetica", Font.BOLD, 16);
	protected static Color VOTING_PAGE_COLOR = new Color(0.5f, 0.5f, 0.5f);
	protected static Color INFORMATION_PAGE_COLOR = Color.LIGHT_GRAY;
	protected static double PADDING = 5;
	protected static double MARGIN = 1;
	protected static double PAGE_WIDTH = 500;

	protected Page page;

	protected ArrayList title;
	protected ArrayList instructions;
	protected ArrayList items;
	protected Color baseColor;
	protected boolean hasFocus = true;
	protected PNode overviewBackground;
	public ButtonNode submitButton;

	public PageNode(Page page) {
		this.page = page;
				
		title = new ArrayList();
		instructions = new ArrayList();
		items = new ArrayList();
		
		Iterator i;

		i = page.getTitle().iterator();
		while (i.hasNext()) {
			PText each = new PText((String)i.next());
			each.setPaint(Color.WHITE);
			each.setFont(TITLE_FONT);
			each.setConstrainWidthToTextWidth(false);
			each.setCenteredText(true);
			addChild(each);
			title.add(each);
		}
		
		i = page.getInstructions().iterator();
		while (i.hasNext()) {
			PText each = new PText((String)i.next());
			each.setPaint(Color.WHITE);
			each.setFont(BallotNode.INSTRUCTIONS_FONT);
			each.setConstrainWidthToTextWidth(false);
			addChild(each);
			instructions.add(each);
		}
		
		i = page.getItems().iterator();
		while (i.hasNext()) {
			ItemNode each = new ItemNode((Item)i.next());
			addChild(each);
			items.add(each);
		}
		
		if (page.getSubmitButtonLabel() != null) {
			submitButton = new ButtonNode(page.getSubmitButtonLabel(), null);
			submitButton.setPickable(false);
			addChild(submitButton);
			submitButton.addInputEventListener(new PBasicInputEventHandler() {
				public void mousePressed(PInputEvent event) {
					submitButton.setPaint(((Color)submitButton.getPaint()).darker());
					event.setHandled(true);
				}
				public void mouseReleased(PInputEvent event) {
					submitButton.setPaint(((Color)submitButton.getPaint()).brighter());
					Controller.CONTROLLER.submitVote();
					event.setHandled(true);
				}
			});
		}
		
		if (page.getItems().size() > 0) {
			setPaint(VOTING_PAGE_COLOR);
		} else {
			setPaint(INFORMATION_PAGE_COLOR);
		}
		
		baseColor = (Color) getPaint();
		
		addInputEventListener(new PBasicInputEventHandler() {
			public void mousePressed(PInputEvent event) {
				setPaint(((Color)getPaint()).darker());
				event.setHandled(true);
			}
			public void mouseReleased(PInputEvent event) {
				setPaint(((Color)getPaint()).brighter());
				Controller.CONTROLLER.setFocusNode(PageNode.this);
				event.setHandled(true);
			}
		});
	}
	
	public Page getPage() {
		return page;
	}
	
	public void performPartyColumnLayout(double width) {
		Iterator i = items.iterator();
		while (i.hasNext()) {
			ItemNode each = (ItemNode) i.next();
			each.performPartyColumnLayout(width);
		}

		layoutItems(width);
	}
	
	public void performOfficeBlockLayout(double width) {
		Iterator i = items.iterator();
		while (i.hasNext()) {
			ItemNode each = (ItemNode) i.next();
			each.performOfficeBlockLayout(width);
		}
		
		layoutItems(width);
	}	
	
	public void layoutItems(double width) {
		int columns = 1;
		double xOffset = PADDING + PageNode.MARGIN;
		double yOffset = PADDING + PageNode.MARGIN;

		Iterator i;

		i = items.iterator();
		while (i.hasNext()) {
			ItemNode each = (ItemNode) i.next();
			if (each.getPageBreak()) {
				columns++;
			}
		}
		
		i = title.iterator();
		while (i.hasNext()) {
			PText each = (PText) i.next();
			each.setWidth((width * columns) + (ItemNode.MARGIN * (columns - 1)));
			each.offset(xOffset, yOffset);
			yOffset += each.getHeight() + ItemNode.MARGIN;
		}		

		yOffset += PageNode.MARGIN;

		i = instructions.iterator();
		while (i.hasNext()) {
			PText each = (PText) i.next();
			each.setWidth((width * columns) + (ItemNode.MARGIN * (columns - 1)));
			each.offset(xOffset, yOffset);
			yOffset += each.getHeight() + ItemNode.MARGIN;
		}

		yOffset += PageNode.MARGIN;

		int size = items.size();
		double startXOffset = xOffset;
		double startYOffset = yOffset;
		double maxXOffset = xOffset;
		
		for (int j = 0; j < size; j++) {
			ItemNode each = (ItemNode) items.get(j);

			if (each.getPageBreak()) {
				xOffset += (width + ItemNode.MARGIN);
				yOffset = startYOffset;
			}
			
			each.offset(xOffset, yOffset);
			yOffset += each.getHeight() + ItemNode.MARGIN;
			if (xOffset > maxXOffset) maxXOffset = xOffset;
		}
		
		yOffset = startYOffset;
		xOffset = maxXOffset;
		
/*		i = items.iterator();
		while (i.hasNext()) {
			ItemNode each = (ItemNode) i.next();
			each.offset(xOffset, yOffset);
			yOffset += each.getHeight() + ItemNode.MARGIN;
		}
*/
		if (submitButton != null) {
			submitButton.buttonLabel.setConstrainWidthToTextWidth(true);
			//submitButton.buttonLabel.setFont(PageNode.TITLE_FONT);
			submitButton.setBounds(submitButton.getUnionOfChildrenBounds(null).inset(0, -10));
			submitButton.setWidth(width * columns);
			submitButton.positionLabel();
			submitButton.offset(xOffset, yOffset - submitButton.getHeight());
			submitButton.centerFullBoundsOnPoint(getFullBoundsReference().getCenterX(), submitButton.getFullBoundsReference().getCenterY());
		}

		getUnionOfChildrenBounds(getBoundsReference());
		getBoundsReference().inset(-(PADDING + PageNode.MARGIN), -(PADDING + PageNode.MARGIN));
		setHasFocus(false);
	}
	
	PNodeFilter filter = new PNodeFilter() {
		public boolean accept(PNode aNode) { return aNode instanceof PText; }
		public boolean acceptChildrenOf(PNode aNode) { return true; }
	};
	
	public void setFaded(boolean faded) {
		if (faded) {
			Iterator i = title.iterator();
			while (i.hasNext()) {
				PText each = (PText) i.next();
				each.setTextPaint(Color.GRAY);
			}

			i = instructions.iterator();
			while (i.hasNext()) {
				PText each = (PText) i.next();
				each.setTextPaint(Color.GRAY);
			}
			
			if (overviewBackground != null) {
				i = overviewBackground.getAllNodes(filter, null).iterator();
				while (i.hasNext()) {
					PText each = (PText) i.next();
					each.setTextPaint(ItemNode.FADED_TEXT_COLOR);
				}
				Controller.schedualNewActivityFor(overviewBackground, overviewBackground.animateToColor(ItemNode.FADED_BACKGROUND_COLOR, Controller.ANIMATION_TIME));
			}

			Controller.schedualNewActivityFor(this, this.animateToColor(baseColor.brighter(), Controller.ANIMATION_TIME));

		} else {
			Iterator i = title.iterator();
			while (i.hasNext()) {
				PText each = (PText) i.next();
				each.setTextPaint(Color.BLACK);
			}

			i = instructions.iterator();
			while (i.hasNext()) {
				PText each = (PText) i.next();
				each.setTextPaint(Color.BLACK);
			}

			if (overviewBackground != null) {
				i = overviewBackground.getAllNodes(filter, null).iterator();
				while (i.hasNext()) {
					PText each = (PText) i.next();
					each.setTextPaint(Color.BLACK);
				}
				Controller.schedualNewActivityFor(overviewBackground, overviewBackground.animateToColor(Color.WHITE, Controller.ANIMATION_TIME));
			}			

			Controller.schedualNewActivityFor(this, this.animateToColor(baseColor, Controller.ANIMATION_TIME));
		}
	}
	
	public void setFullFaded(boolean faded) {
		setFaded(faded);
		
		Iterator i = items.iterator();
		while (i.hasNext()) {
			ItemNode each = (ItemNode) i.next();
			each.setFullFaded(faded);
		}
	}

	public void setHasFocus(boolean hasFocus) {
		if (this.hasFocus != hasFocus) {
			this.hasFocus = hasFocus;

			if (hasFocus) {
				if (overviewBackground != null) {
					removeChild(overviewBackground);
					overviewBackground = null;
				}

				Iterator i = instructions.iterator();
				while (i.hasNext()) {
					PNode each = (PNode) i.next();
					each.setVisible(true);
				}

				if (submitButton != null) {
					submitButton.setVisible(true);					
					submitButton.setPickable(true);
				}
			} else {
				if (overviewBackground == null) {

					overviewBackground = new PNode();
					overviewBackground.setPaint(CandidateNode.BACKGROUND);

					PBounds overviewBounds = new PBounds();

					Iterator i = instructions.iterator();
					while (i.hasNext()) {
						PNode each = (PNode) i.next();
						each.setVisible(false);
						overviewBounds.add(each.getFullBoundsReference());
					}
		
					if (submitButton != null) {
						submitButton.setVisible(false);
						overviewBounds.add(submitButton.getFullBoundsReference());
					}
					
					overviewBackground.setBounds(overviewBounds);
					overviewBackground.setBounds(overviewBackground.getBoundsReference().inset(1, 1));
					overviewBackground.setChildrenPickable(false);

					PNode overview = new PNode();

					PText label = new PText(page.getSemanticLabel());
					label.setFont(ItemNode.OVERVIEW_TITLE_FONT);
					label.setConstrainWidthToTextWidth(false);
					label.setConstrainHeightToTextHeight(true);
					label.setCenteredText(true);
					label.setWidth(overviewBounds.getWidth());
					overview.addChild(label);

					overview.setBounds(overview.getUnionOfChildrenBounds(overview.getBoundsReference()));
					overviewBackground.addChild(overview);
					overview.centerFullBoundsOnPoint(overviewBackground.getBoundsReference().getCenterX(), overviewBackground.getBoundsReference().getCenterY());
					addChild(overviewBackground);
				}
			}
		}
	}

	public void clearBallot() {
		Iterator i = items.iterator();
		while (i.hasNext()) {
			ItemNode each = (ItemNode) i.next();
			each.clearBallot();
		}
	}
		
	public static void loadStyleInformation(Element style) {
		try {
			Element pageLayout = style.getChild("layout");
			PageNode.INFORMATION_PAGE_COLOR = Controller.readColorStyle(style.getAttributeValue("informationpagecolor"));
			PageNode.VOTING_PAGE_COLOR = Controller.readColorStyle(style.getAttributeValue("votingpagecolor"));
			PageNode.MARGIN = pageLayout.getAttribute("margin").getFloatValue();
			PageNode.PADDING = pageLayout.getAttribute("padding").getFloatValue();
			PageNode.PAGE_WIDTH = pageLayout.getAttribute("width").getFloatValue();
			PageNode.TITLE_FONT = Controller.readFontStyle(style.getChild("title"));
		} catch (DataConversionException e) {
			e.printStackTrace();
		}
	}	
}
